#loading the data file
library(xlsx)
vote.data <- read.xlsx(file="vote_catvarclus.xls",header=T,sheetIndex=1)
summary(vote.data)

#active variables for the analysis
vote.active <- subset(vote.data,select=2:7)

#*****************************
#cluster analysis on variables
#*****************************
library(ClustOfVar)
arbre <- hclustvar(X.quali=vote.active)
plot(arbre)

sol <- cutreevar(arbre,k=2,matsim=T)
print(sol)

#***************************************************
#cluster analysis on levels of qualitative variables
#***************************************************

#dummy coding
library(ade4)
disj <- acm.disjonctif(vote.active)

#Dice's index as measure of dissimlarity
#Abdallah & Saporta, page 79
m <- matrix(0,ncol(disj),ncol(disj))
for (j in 1:ncol(disj)){
  for (jprim in 1:ncol(disj)){
    m[j,jprim] <- 0.5 * sum(disj[,j]*(1-disj[,jprim])+disj[,jprim]*(1-disj[,j]))
  }
}

colnames(m) <- colnames(disj)
rownames(m) <- colnames(disj)

#transform the matrix in a dist class
d <- as.dist(sqrt(m))

#cluster analysis on indicator variables
arbre.moda <- hclust(d,method="average")
plot(arbre.moda)

#sorting the height in a decreasing order
v <- sort(arbre.moda$height,decreasing=T)
#plotting
plot(1:length(v),v,type="b")

#******************************
#Varclus from the Hmisc package
#******************************
library(Hmisc)
v <- varclus(as.matrix(disj),type="data.matrix",similarity="bothpos",method="average")
plot(v)

#the used matrix of similarity
round(v$sim,2)

#the call to the hclust() procedure
v$hclust$height

#**********************************************
#How to obtain the same results as in varclus()
#**********************************************

#matrx for the bothpos measure
#cf. varclus documentation (Hmisc)
bp <- matrix(0,ncol(disj),ncol(disj))
for (j in 1:ncol(disj)){
  for (jprim in 1:ncol(disj)){
    bp[j,jprim] <- mean(disj[,j]*disj[,jprim])
  }
}

colnames(bp) <- colnames(disj)
rownames(bp) <- colnames(disj)

#dissimliraty measure from bp
dp <- as.dist(1-bp)

#cluster analysis from the dp dissimilarity matrix
arbre.bp <- hclust(dp,method="average")
plot(arbre.bp)

arbre.bp$height

#********************************************************
#cluster analysis from the coordinates of levels provided
#by the MCA multiple correspondence analysis
#********************************************************

#MCA with the ade4 package
acm <- dudi.coa(disj,scannf=F,nf=2)
acm.coord <- data.frame(acm$co)
rownames(acm.coord) <- colnames(disj)

#graphical representation
plot(acm.coord[,1],acm.coord[,2],type="n",xlim=c(-1.25,1.25),ylim=c(-0.5,4.5),xlab="F1",ylab="F2")
text(acm.coord[,1],acm.coord[,2],labels=rownames(acm.coord),cex=0.75,col="blue")
abline(h=0,v=0)

#distance matrix of the levels of qualitative variables
m.acm <- dist(acm.coord,method="euclidian")

#cluster analysis from the distance matrix m.acm
arbre.acm <- hclust(m.acm,method="average")
plot(arbre.acm)
